# Package metadata
__name__ = "pypiwrap"
__author__ = "Ángel Carias"
__license__ = "MIT"
__version__ = "2.0.0"

# PyPI constants
PYPI_HOST = "https://pypi.org"
USER_AGENT = f"aescarias/pypiwrap {__version__}"

SUPPORTED_SIMPLE_VERSION = (1, 3)
SIMPLE_CONTENT_TYPE = "application/vnd.pypi.simple.v1+json"

INTEGRITY_CONTENT_TYPE = "application/vnd.pypi.integrity.v1+json"
